@@ -52,7 +52,7 @@ import org.apache.myfaces.shared.util._ComponentUtils;
  * @author Manfred Geiler (latest modification by $Author: lu4242 $)
  * @author Thomas Spiegl
  * @author Anton Koinov
- * @version $Revision: 1533195 $ $Date: 2013-10-17 18:21:06 +0000 (Thu, 17 Oct 2013) $
+ * @version $Revision: 1669647 $ $Date: 2015-03-27 18:21:54 +0000 (Fri, 27 Mar 2015) $
  */
 public class HtmlButtonRendererBase
     extends HtmlRenderer
@@ -69,11 +69,11 @@ public class HtmlButtonRendererBase
 
         //super.decode must not be called, because value is handled here
         boolean disabled = isDisabled(facesContext, uiComponent);
-        if (!isReset(uiComponent) && isSubmitted(facesContext, uiComponent) &&
-            !disabled)
+        // MYFACES-3960 Decode, decode client behavior and queue action event at the end
+        boolean activateActionEvent = !isReset(uiComponent) && isSubmitted(facesContext, uiComponent) &&
+            !disabled;
+        if (activateActionEvent)
         {
-            uiComponent.queueEvent(new ActionEvent(uiComponent));
-
             org.apache.myfaces.shared.renderkit.RendererUtils.initPartialValidationAndModelUpdate(
                     uiComponent, facesContext);
         }
@@ -83,6 +83,11 @@ public class HtmlButtonRendererBase
         {
             HtmlRendererUtils.decodeClientBehaviors(facesContext, uiComponent);
         }
+        
+        if (activateActionEvent)
+        {
+            uiComponent.queueEvent(new ActionEvent(uiComponent));
+        }
     }
 
     private static boolean isReset(UIComponent uiComponent)
